	
local BankUI = {
	SelectedIndex = 1,
	SelectedID = nil,
	PlayerIndex = 1,
	RealID_Data = {},
	professioncolors = {
		NONE     = "|cff404040",
		ALT      = "|cffffff00",
		GUILDON  = "|cff40ff40",
		GUILDOF  = "|cff408040",
		FRIENDON = "|cffffffff",
		FRIENDOF = "|cffA0A0A0",
		BNFRNDON = "|cff00ffff",
		BNFRNDOF = "|cff008080",
	},
	tradecolors = {
		["trivial"]   = "|cff808080",
		["easy"]      = "|cff00ff00",
		["medium"]    = "|cffffff00",
		["optimal"]   = "|cffff8000",
		["difficult"] = "|cffff0000",
	},
	linktochattypes = {
		{"Link to:", nil, true},
		{"RAID", nil, false},
		{"PARTY", nil, false},
		{"GUILD", nil, false},
		{"SAY", nil, false},
		{"FRIEND", 1, false},
		{"WHISPERS", 1, false},
		{"REALID", 1, false},
	},
	Link_Data = {
		["Link to:"] = {},
		["RAID"] = {},
		["PARTY"] = {},
		["GUILD"] = {},
		["SAY"] = {},
		["FRIEND"] = {},
		["REALID"] = {},
		["WHISPERS"] = {},
	},
	Item_Type = {
		[BANKGUI_TYPE_CURRENCY] = "Currency",
		[BANKGUI_TYPE_FRIENDS] = "Friend",
		[BANKGUI_TYPE_GLYPHS] = "Glyph (equiped)",
		[BANKGUI_TYPE_FARMS] = "Farm Item",
		[BANKGUI_TYPE_TRADELINK] = "TradeLink",
		["-3"] = "Equiped",
		["-2"] = "Keyring",
		["-1"] = "Bankslot %i/28",
		["0"] = "Backpack",
		["1"] = "Bag 1",
		["2"] = "Bag 2",
		["3"] = "Bag 3",
		["4"] = "Bag 4",
		["5"] = "BankBag 1",
		["6"] = "BankBag 2",
		["7"] = "BankBag 3",
		["8"] = "BankBag 4",
		["9"] = "BankBag 5",
		["10"] = "BankBag 6",
		["11"] = "BankBag 7",
	},
	Types_Data = {
		"Everything",
		"Items",
		"Tradeskills",
		"Profession Links",
		"Farm Reminders",
		"Glyphs",
		"Friends",
		"Currency",
	},
	FACTION_COLORS = {
		[FACTION_ALLIANCE] = { 0.25, 0.25, 0.75 },
		[FACTION_HORDE]    = { 0.75, 0.25, 0.25 },
		[UNKNOWN]          = { 0.5,  0.5,  0.5 },
	},
}


-- Inserts all of the UI elements
function BankGUI:InsertUI()
	-- Add tab button to SocialFrame
	BankGUI.numtab = _G["FriendsFrame"].numTabs + 1
	CreateFrame("Button", "FriendsFrameTab"..BankGUI.numtab, _G["FriendsFrame"], "FriendsFrameBankTabTemplate", BankGUI.numtab)
	PanelTemplates_SetNumTabs(_G["FriendsFrame"], BankGUI.numtab)
	PanelTemplates_UpdateTabs(_G["FriendsFrame"])
	BankGUI.numtab = _G["FriendsFrame"].numTabs

	_G["FriendsFrameTab"..BankGUI.numtab]:Enable()

	if BankListOptions[BankGUI.realmname][1].detach then
		-- Add the window in UIParent
		CreateFrame("Frame", "BankGUIFrame", _G["UIParent"], "BankGUIFrameTemplate")
		BankGUIDetachCheckButton_OnClick()
	else
		-- Add the window in FriendsFrame
		CreateFrame("Frame", "BankGUIFrame", _G["FriendsFrame"], "BankGUIFrameTemplate")
		table.insert(FRIENDSFRAME_SUBFRAMES, "BankGUIFrame")
		BankGUIDetachCheckButton_OnClick()
		FriendsFrame_Update()
	end
end


function BankGUI:SetSelectedBankGUI(index)
	BankUI.SelectedIndex = index
end


function BankGUI:GetSelectedBankGUI()
	return BankUI.SelectedIndex
end


function BankGUI:OnEnter(self)
	if IsAltKeyDown() then return end
	local bankinfo = BankGUI:GetItemByIndex(self:GetID())
	local bankfaction = BankUI.FACTION_COLORS[ UNKNOWN ]
	if not bankinfo then return end
	CloseDropDownMenus()
	if BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION] then
		if bankinfo.type == BANKGUI_TYPE_FRIENDS then
			if bankinfo.faction == nil then bankinfo.faction = UNKNOWN end
		end
		if bankinfo.faction ~= nil then
			bankfaction = BankUI.FACTION_COLORS[ bankinfo.faction ]
		elseif BankGUI.BankFactionColors[ bankinfo.name ] ~= nil then
			if BankGUI.BankFactionColors[ bankinfo.name ].faction ~= nil then
				bankfaction = BankUI.FACTION_COLORS[ BankGUI.BankFactionColors[ bankinfo.name ].faction ]
			else
				bankfaction = { BankGUI.BankFactionColors[ bankinfo.name ].r, BankGUI.BankFactionColors[ bankinfo.name ].g, BankGUI.BankFactionColors[ bankinfo.name ].b }
			end
		end
	end
	GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
	GameTooltip:ClearLines()
	if bankinfo.type == BANKGUI_TYPE_GLYPHS or bankinfo.type == BANKGUI_TYPE_FRIENDS or bankinfo.type == BANKGUI_TYPE_CURRENCY then
		if IsShiftKeyDown() then
			if bankinfo.type == BANKGUI_TYPE_GLYPHS and bankinfo.item ~= nil then
				GameTooltip:SetHyperlink(bankinfo.item)
			end
		else
			if bankinfo.type ~= BANKGUI_TYPE_FRIENDS then GameTooltip:AddDoubleLine("Owner", bankinfo.name, 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3]) end
			GameTooltip:AddDoubleLine(BANKGUI_TYPE, BankUI.Item_Type[bankinfo.type], 1,1,0, 1,1,0.5)
			if bankinfo.type == BANKGUI_TYPE_FRIENDS then
				GameTooltip:AddDoubleLine("", bankinfo.faction, 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3])
			end
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine("Shift-Click", "Put Itemlink in Chatframe", 1,1,0, 1,1,0.5)
			if bankinfo.type == BANKGUI_TYPE_GLYPHS then GameTooltip:AddDoubleLine("Shift-Mouseover","Display ingame Item tooltip", 1,1,0, 1,1,0.5) end
		end
	elseif bankinfo.type == BANKGUI_TYPE_TRADELINK or bankinfo.type == BANKGUI_TYPE_FARMS then
		if bankinfo.type == BANKGUI_TYPE_FARMS then
			local itemName, itemLink = GetItemInfo(bankinfo.item)
			if itemLink then GameTooltip:SetHyperlink(itemLink)
			else GameTooltip:AddLine(bankinfo.item)
			end
			GameTooltip:AddLine(" ")
		end
		GameTooltip:AddDoubleLine(BANKGUI_TYPE, BankUI.Item_Type[bankinfo.type], 1,1,0, 1,1,0.5)
		GameTooltip:AddDoubleLine("Owner", bankinfo.name, 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3])
		GameTooltip:AddLine(" ")
		GameTooltip:AddDoubleLine("Shift-Click", "Put Itemlink in Chatframe", 1,1,0, 1,1,0.5)
		if bankinfo.type == BANKGUI_TYPE_TRADELINK then
			BankUI.professioncolors.GUILDON = BankGUI.guildcolor_on
			BankUI.professioncolors.GUILDOF = BankGUI.guildcolor_of
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine("Color coding:")
			GameTooltip:AddDoubleLine("No match", string.format("%sDark Grey|r", BankUI.professioncolors.NONE), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Alt Character", string.format("%sYellow|r", BankUI.professioncolors.ALT), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Online Guildmember", string.format("%sGreen|r", BankUI.professioncolors.GUILDON), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Offline Guildmember", string.format("%sDark Green|r", BankUI.professioncolors.GUILDOF), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Online Friend", string.format("%sWhite|r", BankUI.professioncolors.FRIENDON), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Offline Friend", string.format("%sGrey|r", BankUI.professioncolors.FRIENDOF), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Online RealID Friend", string.format("%sCyan|r", BankUI.professioncolors.BNFRNDON), 1,1,0, 1,1,0.5)
			GameTooltip:AddDoubleLine("Offline RealID Friend", string.format("%sDark Cyan|r", BankUI.professioncolors.BNFRNDOF), 1,1,0, 1,1,0.5)
		end
	else
		local itemName, itemLink, itemQuality, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(bankinfo.item)
		if itemStackCount == nil then itemStackCount = 1 end
		if itemTexture == nil then
			if string.find(bankinfo.item,"Alchemy:") then itemSubType = "Alchemy"
			elseif string.find(bankinfo.item,"Blacksmithing:") then itemSubType = "Blacksmithing"
			elseif string.find(bankinfo.item,"Enchanting:") or string.find(bankinfo.item,"Enchant ") then itemSubType = "Enchanting"
			elseif string.find(bankinfo.item,"Engineering:") then itemSubType = "Engineering"
			elseif string.find(bankinfo.item,"Jewelcrafting:") then itemSubType = "Jewelcrafting"
			elseif string.find(bankinfo.item,"Leatherworking:") then itemSubType = "Leatherworking"
			elseif string.find(bankinfo.item,"Tailoring:") then itemSubType = "Tailoring"
			elseif string.find(bankinfo.item,"Mining:") then itemSubType = "Mining"
			elseif string.find(bankinfo.item,"Cooking:") then itemSubType = "Cooking"
			elseif string.find(bankinfo.item,"First Aid:") then itemSubType="First Aid"
			end
		end
		if itemSubType == nil then itemSubType = "" end
		if itemType == nil then
			itemType = itemSubType
			itemSubType = ""
		end
		if IsShiftKeyDown() then
			GameTooltip:SetHyperlink(bankinfo.item)
			GameTooltip:AddLine(" ")
		else
			GameTooltip:AddLine(bankinfo.item)
			if not BankListOptions[BankGUI.realmname][1].guild_checked then
				GameTooltip:AddDoubleLine("Owner", bankinfo.name, 1,1,0, bankfaction[1],bankfaction[2],bankfaction[3])
			end
			if itemStackCount ~= 1 or BankListOptions[BankGUI.realmname][1][BANKGUI_TOTALS] == true then GameTooltip:AddDoubleLine(BANKGUI_AMOUNT, format("%i / %i", bankinfo.amount, itemStackCount), 1,1,0, 1,1,0.5) end
			if BankGUI.TypesIndex ~= 4 and bankinfo.bank == true then
				GameTooltip:AddDoubleLine(BANKGUI_TYPE, format("%s,%s",itemType,itemSubType), 1,1,0, 1,1,0.5)
				local bank_level = ""
				if itemLevel and itemMinLevel then
					bank_level = bank_level ..format("%i / %i", itemMinLevel, itemLevel)
				elseif itemLevel then
					bank_level = bank_level ..itemLevel
				elseif itemMinLevel then
					bank_level = bank_level ..itemMinLevel
				end
				if itemQuality then
					if bank_level ~= "" then bank_level = bank_level ..", " end
					bank_level = bank_level .._G["ITEM_QUALITY"..itemQuality.."_DESC"]
				end
				GameTooltip:AddDoubleLine(BANKGUI_LEVEL, bank_level, 1,1,0, 1,1,0.5)
				if BankListOptions[BankGUI.realmname][1].guild_checked then
					if bankinfo.type ~= nil then GameTooltip:AddDoubleLine(BANKGUI_INVENT, "Tab " ..bankinfo.type, 1,1,0,1,1,0.5) end
				else
					if not BankListOptions[BankGUI.realmname][1][BANKGUI_TOTALS] or itemStackCount == 1 then
						if bankinfo.type ~= nil then
							if bankinfo.type == "-1" then
								GameTooltip:AddDoubleLine(BANKGUI_INVENT, string.format(BankUI.Item_Type[bankinfo.type], bankinfo.slot), 1,1,0,1,1,0.5)
							else
								GameTooltip:AddDoubleLine(BANKGUI_INVENT, BankUI.Item_Type[bankinfo.type], 1,1,0,1,1,0.5)
							end
						end
					else
						GameTooltip:AddDoubleLine(BANKGUI_INVENT, "-", 1,1,0,1,1,0.5)
					end
				end
			else
				GameTooltip:AddDoubleLine(BANKGUI_TYPE, format("%s,%s", itemType, itemSubType), 1,1,0, 1,1,0.5)
				if bankinfo.type ~= nil then
					if BankUI.tradecolors[bankinfo.type] then GameTooltip:AddDoubleLine(BANKGUI_LEVEL, format("%s%s|r", BankUI.tradecolors[bankinfo.type], bankinfo.type), 1,1,0, 1,1,0.5)
					else GameTooltip:AddDoubleLine(BANKGUI_LEVEL, bankinfo.type, 1,1,0, 1,1,0.5)
					end
					GameTooltip:AddDoubleLine(BANKGUI_INVENT, "Tradeskill", 1,1,0, 1,1,0.5)
				end
			end
			if (itemEquipLoc and _G[itemEquipLoc]) then
				GameTooltip:AddDoubleLine(BANKGUI_EQUIP, _G[itemEquipLoc], 1,1,0, 1,1,0.5)
			end
			if itemStackCount ~= 1 and BankListOptions[BankGUI.realmname][1][BANKGUI_TOOLTIPS] then
				BankGUI:AddTooltip(GameTooltip, itemName)
			end
			GameTooltip:AddLine(" ")
			GameTooltip:AddDoubleLine("Shift-Mouseover","Display ingame Item tooltip", 1,1,0, 1,1,0.5)
		end
		GameTooltip:AddDoubleLine("Shift-Click", "Put Itemlink in Chatframe", 1,1,0, 1,1,0.5)
		GameTooltip:AddDoubleLine("Control-Click", "Dressup Item", 1,1,0,1,1,0.5)
		GameTooltip:AddDoubleLine("Alt-Click", "Put Itemlink in Farm Item name", 1,1,0, 1,1,0.5)
		GameTooltip:AddDoubleLine("Right-Click", "Link via menu", 1,1,0, 1,1,0.5)
	end
	GameTooltip:Show()
end


function BankGUI:ShowDetails()
	-- get info
	local bankinfo = BankGUI:GetItemByIndex(BankGUI:GetSelectedBankGUI())
	local itemName, itemLink, itemQuality, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(bankinfo.item)
	local itemtx = _G["BankGUIDetailsItemText"]
	local textur = _G["BankGUIDetailsIconTexture"]
	local amount = _G["BankGUIDetailsAmountText"]
	local ilevel = _G["BankGUIDetailsLevelText"]
	local ilocat = _G["BankGUIDetailsLocaText"]
	local iequip = _G["BankGUIDetailsEquiText"]
	local prefix = "Interface\\Icons\\"

	local bankfaction = { 1.0,1.0,1.0 }
	if BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION] then
		if bankinfo.faction ~= nil then
			bankfaction = BankUI.FACTION_COLORS[ bankinfo.faction ]
		elseif BankGUI.BankFactionColors[ bankinfo.name ] ~= nil then
			if BankGUI.BankFactionColors[ bankinfo.name ].faction ~= nil then
				bankfaction = BankUI.FACTION_COLORS[ BankGUI.BankFactionColors[ bankinfo.name ].faction ]
			else
				bankfaction = { BankGUI.BankFactionColors[ bankinfo.name ].r, BankGUI.BankFactionColors[ bankinfo.name ].g, BankGUI.BankFactionColors[ bankinfo.name ].b }
			end
		else
			bankfaction = { 0.25, 0.25, 0.25 }
		end
	end

	-- update details
	if itemLink then itemtx:SetText( itemLink )
	else itemtx:SetText( bankinfo.item ) end
	if itemStackCount == nil then itemStackCount = 1 end
	if itemTexture == nil then itemTexture = GetItemIcon(bankinfo.item) end
	if itemTexture == nil then
		if string.find(bankinfo.item,"Alchemy:") then itemTexture = prefix.."Trade_Alchemy"itemSubType = "Alchemy"
		elseif string.find(bankinfo.item,"Blacksmithing:") then itemTexture = prefix.."Trade_Blacksmithing"itemSubType = "Blacksmithing"
		elseif string.find(bankinfo.item,"Enchanting:") or string.find(bankinfo.item,"Enchant ") then itemTexture = prefix.."Trade_Engraving"itemSubType = "Enchanting"
		elseif string.find(bankinfo.item,"Engineering:") then itemTexture = prefix.."Trade_Engineering"itemSubType = "Engineering"
		elseif string.find(bankinfo.item,"Inscription:") then itemTexture = prefix.."INV_Inscription_Tradeskill01"itemSubType = "Inscription"
		elseif string.find(bankinfo.item,"Jewelcrafting:") then itemTexture = prefix.."INV_Misc_Gem_02"itemSubType = "Jewelcrafting"
		elseif string.find(bankinfo.item,"Leatherworking:") then itemTexture = prefix.."Trade_Leatherworking"itemSubType = "Leatherworking"
		elseif string.find(bankinfo.item,"Tailoring:") then itemTexture = prefix.."Trade_Tailoring"itemSubType = "Tailoring"
		elseif string.find(bankinfo.item,"Mining:") then itemTexture = prefix.."Trade_Mining"itemSubType = "Mining"
		elseif string.find(bankinfo.item,"Cooking:") then itemTexture = prefix.."INV_Misc_Food_15"itemSubType = "Cooking"
		elseif string.find(bankinfo.item,"First Aid:") then itemTexture = prefix.."Spell_Holy_SealOfSacrifice"itemSubType = "First Aid"
		end
	end
	textur:SetTexture(itemTexture)
	if itemSubType == nil then itemSubType = "" end
	if itemType == nil then
		itemType = itemSubType
		itemSubType = ""
	end
	textur:SetTexCoord(0, 1, 0, 1)
	if bankinfo.bank == true then
		if itemStackCount ~= 1 then amount:SetText( format("%i / %i", bankinfo.amount, itemStackCount) )
		else amount:SetText( format("%i", bankinfo.amount) )
		end
		if itemMinLevel and itemQuality then ilevel:SetText( format("%i, %s", itemMinLevel, _G["ITEM_QUALITY"..itemQuality.."_DESC"]) )
		elseif itemMinLevel then ilevel:SetText( "" ..itemMinLevel )
		else ilevel:SetText( "" )
		end
	else
		amount:SetText( "" )
		if bankinfo.type ~= nil then
			if BankUI.tradecolors[bankinfo.type] then ilevel:SetText( format("%s%s|r",BankUI.tradecolors[bankinfo.type],bankinfo.type) )
			elseif bankinfo.type == BANKGUI_TYPE_GLYPHS then ilevel:SetText( "-" )
			else ilevel:SetText( bankinfo.type )
			end
		else ilevel:SetText( "" )
		end
	end
	_G["BankGUIDetailsTypeText"]:SetText( itemType ..", " ..itemSubType )
	if BankListOptions[BankGUI.realmname][1].guild_checked then
		if bankinfo.type ~= nil then ilocat:SetText( "Tab " ..bankinfo.type )
		else ilocat:SetText( "-" )
		end
		if (itemEquipLoc and _G[itemEquipLoc]) then iequip:SetText( _G[itemEquipLoc] )
		else iequip:SetText( "-" )
		end
	else
		_G["BankGUIDetailsNameText"]:SetTextColor(bankfaction[1],bankfaction[2],bankfaction[3])
		_G["BankGUIDetailsNameText"]:SetText( bankinfo.name )
		if bankinfo.bank == true then
			if not BankListOptions[BankGUI.realmname][1][BANKGUI_TOTALS] or itemStackCount == 1 then
				ilocat:SetText( BankUI.Item_Type[bankinfo.type] )
			else
				ilocat:SetText( "-" )
			end
		elseif bankinfo.type == BANKGUI_TYPE_GLYPHS then ilocat:SetText( "Glyph socket" )
		else ilocat:SetText( format("|cffffffff%s|r", "Tradeskill") )
		end
		if (itemEquipLoc and _G[itemEquipLoc]) then iequip:SetText( _G[itemEquipLoc] )
		elseif bankinfo.type == BANKGUI_TYPE_GLYPHS then iequip:SetText( bankinfo.bank )
		else iequip:SetText( "-" )
		end
	end
	BankGUIDetailsFrame:Show()
end


function BankGUI:ResetFilter()
	CloseDropDownMenus()
	BankGUIDetailsFrame:Hide()
	BankGUIFarmFrame:Hide()
	BankGUI.TypesIndex = 1
	BankUI.PlayerIndex = 1
	BankUI.SelectedID = nil
	BankListOptions[BankGUI.realmname][1].player_name = nil
	BankListOptions[BankGUI.realmname][1].player_type = 1
	UIDropDownMenu_SetText(BankGUITypeDropDown, BankUI.Types_Data[BankGUI.TypesIndex])
	UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters")
	BankGUIFilterTextBox:SetText("")
	BankGUIFilterTextBox:SetCursorPosition(0)
	UIDropDownMenu_EnableDropDown(BankGUIPlayerDropDown)
	BankGUIGuildCheckButton_OnClick(BankGUIGuildCheckButton)
end


function BankGUI:UpdateFilter(filter)
	local i, k, matchfound, searchstring, filterstring, bankinfo, name, online, itemName, itemQuality, itemType, itemSubType, itemEquipLoc, subarray, subsubarray, subsubsub = 0, 0, false
	if filter == nil then filterstring = "" else filterstring = string.lower(filter) end
	table.wipe(BankGUI.BankFilter)
	--BankGUI.BankFilter = {}
	table.wipe(BankGUI.BankTotals)
	--BankGUI.BankTotals = {}
	if BankListOptions[BankGUI.realmname][1].guild_checked then
		if not BankListGuildItems[BankGUI.realmname] then BankListGuildItems[BankGUI.realmname] = {} end
		if BankUI.SelectedID and BankUI.SelectedID ~= "" then
			if not BankListGuildItems[BankGUI.realmname][BankUI.SelectedID] then BankListGuildItems[BankGUI.realmname][BankUI.SelectedID] = {} end
			for i = 1, table.getn(BankListGuildItems[BankGUI.realmname][BankUI.SelectedID]) do
				matchfound = false
				searchstring = ""
				bankinfo = {}
				bankinfo.name = BankUI.SelectedID
				for subarray in pairs(BankListGuildItems[BankGUI.realmname][ bankinfo.name ][i]) do
					bankinfo[subarray] = BankListGuildItems[BankGUI.realmname][ bankinfo.name ][i][subarray]
				end
				bankinfo.bank = true
				itemName, _, itemQuality, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(bankinfo.item)
				if itemName == nil then itemName = bankinfo.item end
				if filterstring == "" then matchfound = true
				else
					searchstring = bankinfo.item
					if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
					if itemType then searchstring = searchstring .." " ..itemType end
					if itemSubType then searchstring = searchstring .." " ..itemSubType end
					if itemQuality then searchstring = searchstring .." " .._G["ITEM_QUALITY"..itemQuality.."_DESC"] end
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
				if matchfound then
					if not BankListOptions[BankGUI.realmname][1][BANKGUI_TOTALS] then
						table.insert(BankGUI.BankFilter, bankinfo)
					end
					if not BankGUI.BankTotals[ bankinfo.name ] then BankGUI.BankTotals[ bankinfo.name ] = {} end
					if not BankGUI.BankTotals[ bankinfo.name ][itemName] then
						BankGUI.BankTotals[ bankinfo.name ][itemName] = {}
						for subarray in pairs(bankinfo) do
							BankGUI.BankTotals[ bankinfo.name ][itemName][subarray] = bankinfo[subarray]
						end
					else
						BankGUI.BankTotals[ bankinfo.name ][itemName].amount = BankGUI.BankTotals[ bankinfo.name ][itemName].amount + bankinfo.amount
					end
				end
			end
			BankGUI_SortTotals()
		end
	else
		for i = 1, table.getn(BankListItems[BankGUI.realmname]) do
			if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankGUI.BankFactionColors[ BankListItems[BankGUI.realmname][i].name ] and BankGUI.BankFactionColors[ BankListItems[BankGUI.realmname][i].name ].faction and BankGUI.BankFactionColors[ BankListItems[BankGUI.realmname][i].name ].faction == BankGUI.playerfaction) or (BankListItems[BankGUI.realmname][i].faction and BankListItems[BankGUI.realmname][i].faction == BankGUI.playerfaction) then
				matchfound = false
				searchstring = ""
				bankinfo = {}
				for subarray in pairs(BankListItems[BankGUI.realmname][i]) do
					bankinfo[subarray] = BankListItems[BankGUI.realmname][i][subarray]
				end
				if bankinfo.item then
					itemName, _, itemQuality, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(bankinfo.item)
					if itemName == nil then itemName = bankinfo.item end
					if ( BankGUI.TypesIndex == 1 or (BankGUI.TypesIndex == 2 and bankinfo.bank and BankGUI:IsValidType(bankinfo)) or (BankGUI.TypesIndex == 3 and not bankinfo.bank and BankGUI:IsValidType(bankinfo)) or (BankGUI.TypesIndex == 4 and bankinfo.type == BANKGUI_TYPE_TRADELINK) ) then
						if BankUI.SelectedID == nil then
							if filterstring == "" then matchfound = true
							else
								searchstring = bankinfo.item
								if bankinfo.name ~= nil then searchstring = searchstring ..bankinfo.name end
								if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
								if itemType then searchstring = searchstring .." " ..itemType end
								if itemSubType then searchstring = searchstring .." " ..itemSubType end
								if itemQuality then searchstring = searchstring .." " .._G["ITEM_QUALITY"..itemQuality.."_DESC"] end
								if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
							end
						elseif filterstring == "" then
							if bankinfo.name == BankUI.SelectedID then matchfound = true end
						else
							if string.find(bankinfo.name, BankUI.SelectedID) ~= nil then
								searchstring = bankinfo.item
								if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
								if itemType then searchstring = searchstring .." " ..itemType end
								if itemSubType then searchstring = searchstring .." " ..itemSubType end
								if itemQuality then searchstring = searchstring .." " .._G["ITEM_QUALITY"..itemQuality.."_DESC"] end
								if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
							end
						end
						if matchfound then
							if not BankListOptions[BankGUI.realmname][1][BANKGUI_TOTALS] then
								table.insert(BankGUI.BankFilter, bankinfo)
							end
							if not BankGUI.BankTotals[ bankinfo.name ] then BankGUI.BankTotals[ bankinfo.name ] = {} end
							if not BankGUI.BankTotals[ bankinfo.name ][itemName] then
								BankGUI.BankTotals[ bankinfo.name ][itemName] = {}
								for subarray in pairs(bankinfo) do
									BankGUI.BankTotals[ bankinfo.name ][itemName][subarray] = bankinfo[subarray]
								end
							else
								BankGUI.BankTotals[ bankinfo.name ][itemName].amount = BankGUI.BankTotals[ bankinfo.name ][itemName].amount + bankinfo.amount
							end
						end
					end
				else
					--[[
					print("--------------------------")
					for subarray in pairs(bankinfo) do
						print(subarray..": "..(bankinfo[subarray] or "nil"))
					end
					--]]
				end
			end
		end
		BankGUI_SortTotals()
		BankUI.professioncolors.GUILDON = BankGUI.guildcolor_on
		BankUI.professioncolors.GUILDOF = BankGUI.guildcolor_of
		for i = 1, table.getn(BankGUI.BankFilter) do
			if BankGUI.BankFilter[i].color == BankUI.professioncolors.BNFRNDON then
				BankGUI.BankFilter[i].color = BankUI.professioncolors.BNFRNDOF
			elseif BankGUI.BankFilter[i].color ~= BankUI.professioncolors.ALT then BankGUI.BankFilter[i].color = BankUI.professioncolors.NONE end
			for k = 1, table.getn(BankListPlayers[BankGUI.realmname]) do
				if BankGUI.BankFilter[i].name == BankListPlayers[BankGUI.realmname][k].name then BankGUI.BankFilter[i].color = BankUI.professioncolors.ALT end
			end
			if BankGUI.BankFilter[i].color == BankUI.professioncolors.NONE then
				for k = 1, GetNumGuildMembers(true) do
					name, _, _, _, _, _, _, _, online = GetGuildRosterInfo(k)
					if BankGUI.BankFilter[i].name == name then
						if online then
							BankGUI.BankFilter[i].color = BankUI.professioncolors.GUILDON
						else
							BankGUI.BankFilter[i].color = BankUI.professioncolors.GUILDOF
						end
					end
				end
			end
			if BankGUI.BankFilter[i].color == BankUI.professioncolors.NONE then
				for k = 1, BNGetNumFriends() do
					_, _, _, _, name, _, _, online = GetFriendInfo(k)
					if BankGUI.BankFilter[i].name == name and online then
						BankGUI.BankFilter[i].color = BankUI.professioncolors.BNFRNDON
					end
				end
			end
			if BankGUI.BankFilter[i].color == BankUI.professioncolors.NONE then
				for k = 1, GetNumFriends() do
					name, _, _, _, online = GetFriendInfo(k)
					if BankGUI.BankFilter[i].name == name then
						if online then
							BankGUI.BankFilter[i].color = BankUI.professioncolors.FRIENDON
						else
							BankGUI.BankFilter[i].color = BankUI.professioncolors.FRIENDOF
						end
					end
				end
			end
		end
		if ( BankGUI.TypesIndex == 1 or BankGUI.TypesIndex == 5 ) then
			for i = 1, table.getn(BankListFarms[BankGUI.realmname]) do
			  if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankGUI.BankFactionColors[ BankListFarms[BankGUI.realmname][i].name ] and BankGUI.BankFactionColors[ BankListFarms[BankGUI.realmname][i].name ].faction and BankGUI.BankFactionColors[ BankListFarms[BankGUI.realmname][i].name ].faction == BankGUI.playerfaction) then
				bankinfo = {}
				matchfound = false
				searchstring = ""
				bankinfo.name = BankListFarms[BankGUI.realmname][i].name
				bankinfo.item = BankListFarms[BankGUI.realmname][i].item
				bankinfo.amount = BankListFarms[BankGUI.realmname][i].amount
				bankinfo.date = BankListFarms[BankGUI.realmname][i].date
				bankinfo.type = BANKGUI_TYPE_FARMS
				itemName, _, itemQuality, _, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(bankinfo.item)
				if BankUI.SelectedID == nil then
					if filterstring == "" then matchfound = true
					else
						if itemName == nil then itemName = bankinfo.item end
						searchstring = bankinfo.name ..bankinfo.item
						if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
						if itemType then searchstring = searchstring .." " ..itemType end
						if itemSubType then searchstring = searchstring .." " ..itemSubType end
						if itemQuality then searchstring = searchstring .." " .._G["ITEM_QUALITY"..itemQuality.."_DESC"] end
						if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
					end
				elseif filterstring == "" then
					if bankinfo.name == BankUI.SelectedID then matchfound = true end
				else
					if bankinfo.name == BankUI.SelectedID then
						searchstring = bankinfo.item
						if itemEquipLoc and _G[itemEquipLoc] then searchstring = searchstring .." " ..string.lower(_G[itemEquipLoc]) end
						if itemType then searchstring = searchstring .." " ..itemType end
						if itemSubType then searchstring = searchstring .." " ..itemSubType end
						if itemQuality then searchstring = searchstring .." " .._G["ITEM_QUALITY"..itemQuality.."_DESC"] end
						if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
					end
				end
				if matchfound == true then table.insert(BankGUI.BankFilter, bankinfo) end
			  end
			end
		end
		if ( BankGUI.TypesIndex == 1 or BankGUI.TypesIndex == 6 ) then
			for i = 1, table.getn(BankListGlyphs[BankGUI.realmname]) do
			  if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankGUI.BankFactionColors[ BankListGlyphs[BankGUI.realmname][i].name ] and BankGUI.BankFactionColors[ BankListGlyphs[BankGUI.realmname][i].name ].faction and BankGUI.BankFactionColors[ BankListGlyphs[BankGUI.realmname][i].name ].faction == BankGUI.playerfaction) then
				bankinfo = {}
				matchfound = false
				searchstring = ""
				bankinfo.name = BankListGlyphs[BankGUI.realmname][i].name
				bankinfo.spec = BankListGlyphs[BankGUI.realmname][i].spec
				bankinfo.item = BankListGlyphs[BankGUI.realmname][i].item
				if not BankListGlyphs[BankGUI.realmname][i].glyp then BankListGlyphs[BankGUI.realmname][i].glyp = 0 end
				bankinfo.glyp = BankListGlyphs[BankGUI.realmname][i].glyp
				bankinfo.slot = BankListGlyphs[BankGUI.realmname][i].slot
				bankinfo.type = BANKGUI_TYPE_GLYPHS
				if BankUI.SelectedID == nil then
					if filterstring == "" then matchfound = true
					else
						searchstring = bankinfo.name ..bankinfo.item
						if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
					end
				elseif filterstring == "" then
					if bankinfo.name == BankUI.SelectedID then matchfound = true end
				else
					if bankinfo.name == BankUI.SelectedID then
						searchstring = bankinfo.item
						if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
					end
				end
				if matchfound == true then table.insert(BankGUI.BankFilter, bankinfo) end
			  end
			end
		end
		if ( BankGUI.TypesIndex == 1 or BankGUI.TypesIndex == 7 ) then
			for i = 1, table.getn(BankListFriends[BankGUI.realmname]) do
			  if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankListFriends[BankGUI.realmname][i].faction and BankListFriends[BankGUI.realmname][i].faction == BankGUI.playerfaction) then
				bankinfo = {}
				matchfound = false
				searchstring = ""
				bankinfo.name = BankListFriends[BankGUI.realmname][i].name
				bankinfo.item = BankListFriends[BankGUI.realmname][i].name
				bankinfo.level = BankListFriends[BankGUI.realmname][i].level
				bankinfo.class = BankListFriends[BankGUI.realmname][i].class
				bankinfo.note = BankListFriends[BankGUI.realmname][i].note
				bankinfo.faction = BankListFriends[BankGUI.realmname][i].faction
				bankinfo.type = BANKGUI_TYPE_FRIENDS
				if filterstring == "" then matchfound = true
				else
					searchstring = bankinfo.name
					if bankinfo.class ~= nil then searchstring = searchstring ..bankinfo.class end
					if bankinfo.note ~= nil then searchstring = searchstring ..bankinfo.note end
					if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
				end
				if matchfound == true then table.insert(BankGUI.BankFilter, bankinfo) end
			  end
			end
		end
		if ( BankGUI.TypesIndex == 1 or BankGUI.TypesIndex == 8 ) then
			for i = 1, table.getn(BankListCurrency[BankGUI.realmname]) do
			  if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankGUI.BankFactionColors[ BankListCurrency[BankGUI.realmname][i].name ] and BankGUI.BankFactionColors[ BankListCurrency[BankGUI.realmname][i].name ].faction and BankGUI.BankFactionColors[ BankListCurrency[BankGUI.realmname][i].name ].faction == BankGUI.playerfaction) then
				bankinfo = {}
				matchfound = false
				searchstring = ""
				bankinfo.name = BankListCurrency[BankGUI.realmname][i].name
				if bankinfo.name == nil then bankinfo.name = "" end
				bankinfo.item = BankListCurrency[BankGUI.realmname][i].item
				bankinfo.amount = BankListCurrency[BankGUI.realmname][i].amount
				bankinfo.type = BANKGUI_TYPE_CURRENCY
				if BankUI.SelectedID == nil then
					if filterstring == "" then matchfound = true
					else
						searchstring = bankinfo.name ..bankinfo.item
						if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
					end
				elseif filterstring == "" then
					if bankinfo.name == BankUI.SelectedID then matchfound = true end
				else
					if bankinfo.name == BankUI.SelectedID then
						searchstring = bankinfo.item
						if string.find( string.lower(searchstring), filterstring ) then matchfound = true end
					end
				end
				if matchfound == true then table.insert(BankGUI.BankFilter, bankinfo) end
			  end
			end
		end
	end
	BankGUIDetailsFrame:Hide()
	BankGUI:UpdateUI()
end


function BankGUI:OnEditBoxMouseUp(self, button)
	if button == "RightButton" then self:HighlightText() end
end


function BankGUIDetachCheckButton_OnClick(bank_showframe)
	CloseDropDownMenus()
	BankListOptions[BankGUI.realmname][1].detach = BankGUIDetachCheckButton:GetChecked()--true --BankGUIDetachCheckButton:GetChecked()
	if BankListOptions[BankGUI.realmname][1].detach then
		if FRIENDSFRAME_SUBFRAMES[table.getn(FRIENDSFRAME_SUBFRAMES)] == "BankGUIFrame" then table.remove(FRIENDSFRAME_SUBFRAMES) end
		BankGUIFrame:SetParent("UIParent")
		BankGUIFrame:SetFrameStrata("DIALOG")
		BankGUICloseButton:Show()
		BankGUIFrame:SetMovable(true)
		BankGUIFrame:EnableMouse(true)
		BankGUIFrame:RegisterForDrag("LeftButton")
		BankGUIFrame:ClearAllPoints()
		if BankListOptions[BankGUI.realmname][1]["pos"] then
			BankGUIFrame:SetPoint(BankListOptions[BankGUI.realmname][1]["pos"]["point"], "UIParent", BankListOptions[BankGUI.realmname][1]["pos"]["relativePoint"], BankListOptions[BankGUI.realmname][1]["pos"]["xOfs"], BankListOptions[BankGUI.realmname][1]["pos"]["yOfs"])
		else
			BankGUIFrame:SetPoint("TOPLEFT", "UIParent", "CENTER", -(BankGUIFrame:GetWidth()/2), (BankGUIFrame:GetHeight()/2))
		end
		if bank_showframe == nil then BankGUIFrame:Show() end
	else
		if FRIENDSFRAME_SUBFRAMES[table.getn(FRIENDSFRAME_SUBFRAMES)] ~= "BankGUIFrame" then table.insert(FRIENDSFRAME_SUBFRAMES, "BankGUIFrame") end
		BankGUIFrame:SetParent("FriendsFrame")
		BankGUIFrame:SetFrameStrata("HIGH")
		BankGUICloseButton:Hide()
		BankGUIFrame:SetMovable(false)
		BankGUIFrame:EnableMouse(false)
		BankGUIFrame:RegisterForDrag()
		BankGUIFrame:ClearAllPoints()
		BankGUIFrame:SetPoint("BOTTOMLEFT", _G["FriendsFrame"], "BOTTOMLEFT", -12, -3)
	end
	BankGUI:SetBankFrameBackGround()
	BankGUI:CheckChildFrames()
end


function BankGUIOptionsButton_OnEnter()
	BankGUIDetailsFrame:Hide()
	BankGUIFarmFrame:Hide()
	BankGUIOptionsDropDown.initialize = BankGUIOptionsDropDown_Initialize
	BankGUIOptionsDropDown.displayMode = "MENU"
	ToggleDropDownMenu(1, nil, BankGUIOptionsDropDown, "BankGUIOptionsButton", 0, 0)
end


function BankGUIOptionsDropDown_Initialize(self, level)
	local optionsmenuinfo = UIDropDownMenu_CreateInfo()
	local i, subarray, subsubarray
	if level == nil or level == 1 then
		i = 1
		for subarray in pairs(BankGUI.BANKGUI_OPTIONS_MENU) do
			if subarray == BANKGUI_KEEPTRACKOF or string.find(subarray, "Sort Order") ~= nil then
				if i == 2 then subarray = BANKGUI_SORTTITLE1 end
				if i == 3 then subarray = BANKGUI_SORTTITLE2 end
				if i == 4 then subarray = BANKGUI_SORTTITLE3 end
				optionsmenuinfo.text = subarray
				optionsmenuinfo.value = { ["Level1_Key"] = subarray }
				optionsmenuinfo.func = nil
				optionsmenuinfo.owner = self:GetParent()
				optionsmenuinfo.checked = nil
				optionsmenuinfo.hasArrow = true
				optionsmenuinfo.nested = 1
				optionsmenuinfo.notCheckable = true
				optionsmenuinfo.isTitle = false
				UIDropDownMenu_AddButton(optionsmenuinfo, level)
				i = i + 1
			end
		end
		for i = 1, table.getn(BankGUI.BANKGUI_OPTIONS_MENU) do
			optionsmenuinfo.text = BankGUI.BANKGUI_OPTIONS_MENU[i].name
			optionsmenuinfo.value = i
			optionsmenuinfo.func = BankGUIOptionsDropDown_Select
			optionsmenuinfo.owner = self:GetParent()
			if BankGUI.BANKGUI_OPTIONS_MENU[i]["checkable"] then
				optionsmenuinfo.checked = BankListOptions[BankGUI.realmname][1][ BankGUI.BANKGUI_OPTIONS_MENU[i].name ]
				optionsmenuinfo.notCheckable = false
				optionsmenuinfo.isNotRadio = BankGUI.BANKGUI_OPTIONS_MENU[i]["isNotRadio"]
			else
				optionsmenuinfo.checked = nil
				optionsmenuinfo.notCheckable = true
			end
			optionsmenuinfo.hasArrow = false
			optionsmenuinfo.nested = 0
			optionsmenuinfo.isTitle = false
			UIDropDownMenu_AddButton(optionsmenuinfo, level)
		end
	elseif level == 2 then
		subarray = UIDROPDOWNMENU_MENU_VALUE["Level1_Key"]
		for subsubarray in pairs(BankGUI.BANKGUI_OPTIONS_MENU[subarray]) do
			optionsmenuinfo.text = subsubarray
			optionsmenuinfo.value = { ["Level1_Key"] = subarray, ["Sublevel_Key"] = subsubarray }
			optionsmenuinfo.func = BankGUIOptionsDropDown_Select
			optionsmenuinfo.owner = self:GetParent()
			if subarray == BANKGUI_KEEPTRACKOF then
				optionsmenuinfo.isNotRadio = false
				optionsmenuinfo.checked = BankListOptions[BankGUI.realmname][1][BANKGUI_KEEPTRACKOF][ subsubarray ]
			elseif string.find(subarray, "Sort Order") ~= nil then
				optionsmenuinfo.isNotRadio = true
				if subsubarray == BankListOptions[BankGUI.realmname][1][subarray].name then
					optionsmenuinfo.checked = true
				else
					optionsmenuinfo.checked = false
				end
			end
			optionsmenuinfo.hasArrow = false
			optionsmenuinfo.notCheckable = false
			optionsmenuinfo.isTitle = false
			optionsmenuinfo.disabled = nil
			UIDropDownMenu_AddButton(optionsmenuinfo, level)
		end
	end
end


local function BankGUI_ShowColorPicker(r, g, b, a, myColorPickerCallback)
	ColorPickerFrame.func, ColorPickerFrame.opacityFunc, ColorPickerFrame.cancelFunc = myColorPickerCallback, myColorPickerCallback, myColorPickerCallback
	ColorPickerFrame:SetColorRGB(r, g, b)
	ColorPickerFrame.hasOpacity, ColorPickerFrame.opacity = ( a ~= nil ), a
	ColorPickerFrame.previousValues = {r, g, b, a}
	ColorPickerFrame:Hide() -- Need to run the OnShow handler.
	ColorPickerFrame:Show()
end


local function BankGUI_BarColorCallback(restore)
	local newR, newG, newB, newA
	if restore then
		newR, newG, newB, newA = unpack(restore)
	else
		newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
	end
	BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ].r, BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ].g, BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ].b = newR, newG, newB
	if newA ~= nil then BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ]["a"] = newA end
	BankGUI:SetBankGUIBarColor()
end


local function BankGUI_BackColorCallback(restore)
	local newR, newG, newB, newA
	if restore then
		newR, newG, newB, newA = unpack(restore)
	else
		newA, newR, newG, newB = OpacitySliderFrame:GetValue(), ColorPickerFrame:GetColorRGB()
	end
	BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ].r, BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ].g, BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ].b, BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ]["a"] = newR, newG, newB, newA
	if newA ~= nil then BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ]["a"] = newA end
	BankGUI:SetBankGUIBackgroundColor()
end


local bank_option_value = ""

function BankGUIOptionsDropDown_Select(self)
	CloseDropDownMenus()
	if tonumber(self.value) ~= nil and BankGUI.BANKGUI_OPTIONS_MENU[self.value] and BankGUI.BANKGUI_OPTIONS_MENU[self.value].name ~= nil then
		if BankGUI.BANKGUI_OPTIONS_MENU[self.value].name == BANKGUI_BARCOLOR then
			BankGUI_ShowColorPicker(BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ].r, BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ].g, BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ].b, BankListOptions[BankGUI.realmname][1][ BANKGUI_BARCOLOR ]["a"], BankGUI_BarColorCallback)
		elseif BankGUI.BANKGUI_OPTIONS_MENU[self.value].name == BANKGUI_BACKCOLOR then
			BankGUI_ShowColorPicker(BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ].r, BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ].g, BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ].b, BankListOptions[BankGUI.realmname][1][ BANKGUI_BACKCOLOR ]["a"], BankGUI_BackColorCallback)
		elseif BankGUI.BANKGUI_OPTIONS_MENU[self.value].name == BANKGUI_RESETCOLORS then
			BankGUI:ResetBankGUIBarColor()
			BankGUI:ResetBankGUIBackgroundColor()
			BankGUI:SetBankGUIBarColor()
			BankGUI:SetBankGUIBackgroundColor()
		else
			BankListOptions[BankGUI.realmname][1][ BankGUI.BANKGUI_OPTIONS_MENU[self.value].name ] = not BankListOptions[BankGUI.realmname][1][ BankGUI.BANKGUI_OPTIONS_MENU[self.value].name ]
			if BankGUI.BANKGUI_OPTIONS_MENU[self.value].name == BANKGUI_FACTION then
				BankUI.PlayerIndex = 1
				BankListOptions[BankGUI.realmname][1].player_name = nil
				UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters")
				BankGUIGuildCheckButton_OnClick(BankGUIGuildCheckButton)
				BankGUI:UpdateStock()
				BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
			elseif BankGUI.BANKGUI_OPTIONS_MENU[self.value].name == BANKGUI_TOTALS then
				BankGUI:UpdateStock()
				BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
			end
		end
		BankGUI:SetBankFrameBackGround()
	elseif UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ~= nil then
		local i, subarray
		i = 1
		for subarray in pairs(BankGUI.BANKGUI_OPTIONS_MENU[ UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ]) do
			if i == self:GetID() then bank_option_value = subarray end
			i = i + 1
		end
		if UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] == BANKGUI_KEEPTRACKOF then
			BankListOptions[BankGUI.realmname][1][BANKGUI_KEEPTRACKOF][bank_option_value] = not BankListOptions[BankGUI.realmname][1][BANKGUI_KEEPTRACKOF][bank_option_value]
			if BankListOptions[BankGUI.realmname][1][BANKGUI_KEEPTRACKOF][bank_option_value] == true then return end
			StaticPopupDialogs["DELETE_ALL"] = {
				text = "|cffffff00Do you want to delete all saved info of type |r%s|cffffff00 ?\n\n( NOTE: This is for all Characters on this account. )|r",
				button1 = "Yes",
				button2 = "No",
				OnAccept = function()
					BankGUIOptionsButton_OnYes()
				end,
				timeout = 0,
				whileDead = 1,
				showAlert = 1,
				hideOnEscape = 1
			}
			StaticPopup_Show("DELETE_ALL", bank_option_value)
		elseif UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ~= nil and string.find(UIDROPDOWNMENU_MENU_VALUE["Level1_Key"], "Sort Order") ~= nil then
			BankListOptions[BankGUI.realmname][1][ UIDROPDOWNMENU_MENU_VALUE["Level1_Key"] ].name = bank_option_value
		end
		BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
	end
end


function BankGUIOptionsButton_OnYes()
	local i, bankinfo, bank_keepremoving = 1, nil, true
	if bank_option_value == BANKGUI_KEEPGUILD then
		BankListGuildItems[BankGUI.realmname] = {}
	elseif bank_option_value == BANKGUI_KEEPITEMS then
		if table.getn(BankListItems[BankGUI.realmname]) ~= 0 then
			while bank_keepremoving do
				bankinfo = BankListItems[BankGUI.realmname][i]
				if bankinfo.bank and BankGUI:IsValidType(bankinfo) then
					table.remove(BankListItems[BankGUI.realmname], i)
					i = 1
				else
					i = i + 1
				end	
				if i > table.getn(BankListItems[BankGUI.realmname]) then bank_keepremoving = false end
			end
		end
	elseif bank_option_value == BANKGUI_KEEPTRADE then
		if table.getn(BankListItems[BankGUI.realmname]) ~= 0 then
			while bank_keepremoving do
				bankinfo = BankListItems[BankGUI.realmname][i]
				if string.find(bankinfo.item,"|Henchant") ~= nil then
					table.remove(BankListItems[BankGUI.realmname], i)
					i = 1
				else
					i = i + 1
				end
				if i > table.getn(BankListItems[BankGUI.realmname]) then bank_keepremoving = false end
			end
		end
	elseif bank_option_value == BANKGUI_KEEPPROFS then
		if table.getn(BankListItems[BankGUI.realmname]) ~= 0 then
			while bank_keepremoving do
				bankinfo = BankListItems[BankGUI.realmname][i]
				if bankinfo.type == BANKGUI_TYPE_TRADELINK then
					table.remove(BankListItems[BankGUI.realmname], i)
					i = 1
				else
					i = i + 1
				end
				if i > table.getn(BankListItems[BankGUI.realmname]) then bank_keepremoving = false end
			end
		end
	elseif bank_option_value == BANKGUI_KEEPGLYPS then
		BankListGlyphs[BankGUI.realmname] = {}
	elseif bank_option_value == BANKGUI_KEEPFRIEND then
		BankListFriends[BankGUI.realmname] = {}
	elseif bank_option_value == BANKGUI_KEEPCURR then
		BankListCurrency[BankGUI.realmname] = {}
	end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
end


function BankGUIGuildCheckButton_OnClick(self)
	local i
	CloseDropDownMenus()
	BankListOptions[BankGUI.realmname][1].guild_checked = self:GetChecked()
	BankUI.PlayerIndex = 1
	if BankListOptions[BankGUI.realmname][1].guild_checked then
		BankGUIPlayerPortrait:SetTexture("Interface\\EncounterJournal\\UI-EncounterJournalTextures")
		BankGUIPlayerPortrait:SetTexCoord(0.63281250, 0.72656250, 0.61816406, 0.66015625)
		BankGUI.TypesIndex = 1
		BankUI.SelectedID = BankListOptions[BankGUI.realmname][1].guild_name
		BankGUIPlayerTitle:SetText("Guild")
		UIDropDownMenu_DisableDropDown(BankGUITypeDropDown)
		for i = 1, BankGUI:GetNumGuilds() do
			if BankListOptions[BankGUI.realmname][1].guild_name and BankGUI:GetGuild(i) == BankListOptions[BankGUI.realmname][1].guild_name then BankUI.PlayerIndex = i end
		end
	else
		BankGUIPlayerPortrait:SetTexCoord(0, 1, 0, 1)
		SetPortraitTexture(BankGUIPlayerPortrait, "player")
		if BankListOptions[BankGUI.realmname][1].player_name then BankUI.SelectedID = BankListOptions[BankGUI.realmname][1].player_name
		else BankUI.SelectedID = nil end
		if BankListOptions[BankGUI.realmname][1].player_type then BankGUI.TypesIndex = BankListOptions[BankGUI.realmname][1].player_type end
		BankGUIPlayerTitle:SetText("Char")
		UIDropDownMenu_EnableDropDown(BankGUITypeDropDown)
		for i = 1, table.getn(BankListPlayers[BankGUI.realmname]) do
			if BankUI.SelectedID and BankListPlayers[BankGUI.realmname][i].name == BankUI.SelectedID then BankUI.PlayerIndex = (i + 1) end
		end
	end
	UIDropDownMenu_Initialize(BankGUIPlayerDropDown, BankGUIPlayerDropDown_Initialize)
	if BankListOptions[BankGUI.realmname][1].guild_checked then
		BankUI.SelectedID = BankGUI:GetGuild(BankUI.PlayerIndex)
		UIDropDownMenu_SetText(BankGUITypeDropDown, BankUI.Types_Data[1])
		UIDropDownMenu_SetText(BankGUIPlayerDropDown, BankUI.SelectedID)
		if BankUI.SelectedID == "" then BankGUIDeleteButton:Disable()
		else BankGUIDeleteButton:Enable() end
		BankGUIDeleteButton:SetText(BANKGUI_DELETES)
	else
		UIDropDownMenu_SetText(BankGUITypeDropDown, BankUI.Types_Data[BankGUI.TypesIndex])
		if BankUI.PlayerIndex == 1 then
			UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters")
			BankUI.SelectedID = nil
			BankListOptions[BankGUI.realmname][1].player_name = nil
			if BankGUI.TypesIndex == 4 or BankGUI.TypesIndex == 5 or BankGUI.TypesIndex == 7 then BankGUIDeleteButton:Enable()
			else BankGUIDeleteButton:Disable() end
		else
			BankUI.SelectedID = BankListPlayers[BankGUI.realmname][BankUI.PlayerIndex-1].name
			UIDropDownMenu_SetText(BankGUIPlayerDropDown, BankUI.SelectedID)
			BankGUIDeleteButton:Enable()
		end
		if BankGUI.TypesIndex == 6 or BankGUI.TypesIndex == 8 then
			BankGUIDeleteButton:Disable()
		end
		if BankGUI.TypesIndex == 4 or BankGUI.TypesIndex == 5 or BankGUI.TypesIndex == 7 then BankGUIDeleteButton:SetText(BANKGUI_DELETE)
		else BankGUIDeleteButton:SetText(BANKGUI_DELETES)
		end
		if BankGUI.TypesIndex == 7 then
			UIDropDownMenu_DisableDropDown(BankGUIPlayerDropDown)
		else
			UIDropDownMenu_EnableDropDown(BankGUIPlayerDropDown)
		end
	end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
end


function BankGUIDeleteButton_OnClick()
	CloseDropDownMenus()
	if table.getn(BankGUI.BankFilter) == 0 then return end
	StaticPopupDialogs["DELETE_CHARACTER"] = {
		text = "|cffffff00Are you sure you want to delete all items of|r %s|cffffff00 from the list ?\n\n( NOTE:|r %s|cffffff00 will also be removed from the dropdown menu's )|r",
		button1 = "Yes",
		button2 = "No",
		OnAccept = function()
			BankGUIDeleteButton_OnYes()
		end,
		timeout = 0,
		whileDead = 1,
		which = "",
		showAlert = 1,
		hideOnEscape = 1
	}
	if BankGUI.TypesIndex == 4 or BankGUI.TypesIndex == 5 then
		StaticPopupDialogs["DELETE_CHARACTER"].text = "|cffffff00Are you sure you want to delete|r %s's %s|cffffff00 from the list ?|r"
		StaticPopup_Show("DELETE_CHARACTER", BankGUI.BankFilter[BankUI.SelectedIndex].name, BankGUI.BankFilter[BankUI.SelectedIndex].item)
	elseif BankGUI.TypesIndex == 7 then
		StaticPopupDialogs["DELETE_CHARACTER"].text = "|cffffff00Are you sure you want to delete|r %s|cffffff00 from the list ?|r"
		StaticPopup_Show("DELETE_CHARACTER", BankGUI.BankFilter[BankUI.SelectedIndex].name)
	else StaticPopup_Show("DELETE_CHARACTER", BankUI.SelectedID, BankUI.SelectedID) end
end


function BankGUIDeleteButton_OnYes()
  local i, itemname, bank_itemindexbyname
  CloseDropDownMenus()
  BankGUIDetailsFrame:Hide()
  if BankListOptions[BankGUI.realmname][1].guild_checked then
	for i = 1, BankGUI:GetNumGuilds() do
		if BankListGuildInfo[BankGUI.realmname][i] and BankListGuildInfo[BankGUI.realmname][i].name == BankUI.SelectedID then table.remove(BankListGuildInfo[BankGUI.realmname], i) end
	end
	if not BankListGuildItems[BankGUI.realmname] then return end
	if not BankListGuildItems[BankGUI.realmname][BankUI.SelectedID] then return end
	--[[
	while BankListGuildItems[BankGUI.realmname][BankUI.SelectedID][1] do
		table.remove( BankListGuildItems[BankGUI.realmname][BankUI.SelectedID] )
	end
	--]]
	table.wipe(BankListGuildItems[BankGUI.realmname][BankUI.SelectedID])
	if BankListOptions[BankGUI.realmname][1].guild_name and BankListOptions[BankGUI.realmname][1].guild_name == BankUI.SelectedID then
		BankListOptions[BankGUI.realmname][1].guild_name = BankGUI:GetGuild(1)
		BankUI.SelectedID = BankListOptions[BankGUI.realmname][1].guild_name
	end
  else
	if BankGUI.TypesIndex == 4 then
		for i = 1, table.getn(BankListItems[BankGUI.realmname]) do
			if BankListItems[BankGUI.realmname][i] and BankListItems[BankGUI.realmname][i]["type"] == BANKGUI_TYPE_TRADELINK and BankListItems[BankGUI.realmname][i].name == BankGUI.BankFilter[BankUI.SelectedIndex].name and BankListItems[BankGUI.realmname][i].item == BankGUI.BankFilter[BankUI.SelectedIndex].item then table.remove(BankListItems[BankGUI.realmname], i) end
		end
	elseif BankGUI.TypesIndex == 5 then
		for i = 1, table.getn(BankListFarms[BankGUI.realmname]) do
			if BankListFarms[BankGUI.realmname][i] and BankListFarms[BankGUI.realmname][i].name == BankGUI.BankFilter[BankUI.SelectedIndex].name and BankListFarms[BankGUI.realmname][i].item == BankGUI.BankFilter[BankUI.SelectedIndex].item and BankListFarms[BankGUI.realmname][i].amount == BankGUI.BankFilter[BankUI.SelectedIndex].amount then
				itemname = GetItemInfo(BankGUI.BankFilter[BankUI.SelectedIndex].item)
				if BankGUI.BankListStock[itemname] and BankGUI.BankListStock[itemname].farm then
					BankGUI.BankListStock[itemname].farm.amount = BankGUI.BankListStock[itemname].farm.amount - BankListFarms[BankGUI.realmname][i].amount
					if BankGUI.BankListStock[itemname].farm.amount <= 0 then BankGUI.BankListStock[itemname].farm = nil end
				end
				table.remove(BankListFarms[BankGUI.realmname], i)
				break
			end
		end
	elseif BankGUI.TypesIndex == 7 then
		for i = 1, table.getn(BankListFriends[BankGUI.realmname]) do
			if BankListFriends[BankGUI.realmname][i] and BankListFriends[BankGUI.realmname][i].name == BankGUI.BankFilter[BankUI.SelectedIndex].name then
				table.remove(BankListFriends[BankGUI.realmname], i)
				break
			end
		end
	else
		while true do
			bank_itemindexbyname = BankGUI:GetBankItemIndexByName(BankUI.SelectedID)
			if bank_itemindexbyname == 0 then break end
			table.remove( BankListItems[BankGUI.realmname], bank_itemindexbyname )
		end
		for i = 1, table.getn(BankListPlayers[BankGUI.realmname]) do
			if BankListPlayers[BankGUI.realmname][i] and BankListPlayers[BankGUI.realmname][i].name == BankUI.SelectedID then table.remove(BankListPlayers[BankGUI.realmname], i) end
		end
	end
  end
  BankGUIGuildCheckButton_OnClick(BankGUIGuildCheckButton)
  --BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
end


function BankGUITypeDropDown_Initialize(self)
	local i
	local typemenuinfo = UIDropDownMenu_CreateInfo()
	for i = 1, table.getn(BankUI.Types_Data) do
		typemenuinfo.text = BankUI.Types_Data[i]
		typemenuinfo.value = BankUI.Types_Data[i]
		typemenuinfo.func = BankGUITypeDropDown_Select
		typemenuinfo.owner = self:GetParent()
		typemenuinfo.checked = nil
		typemenuinfo.notCheckable = true
		typemenuinfo.hasArrow = false
		UIDropDownMenu_AddButton(typemenuinfo)
 	end
end


function BankGUITypeDropDown_Select(self)
	BankGUI.TypesIndex = self:GetID()
	if not BankListOptions[BankGUI.realmname][1].guild_checked then
		UIDropDownMenu_SetText(BankGUITypeDropDown, BankUI.Types_Data[BankGUI.TypesIndex])
		BankListOptions[BankGUI.realmname][1].player_type = self:GetID()
		if BankGUI.TypesIndex == 7 then
			UIDropDownMenu_DisableDropDown(BankGUIPlayerDropDown)
		else
			UIDropDownMenu_EnableDropDown(BankGUIPlayerDropDown)
		end
		if BankGUI.TypesIndex == 4 or BankGUI.TypesIndex == 5 or BankGUI.TypesIndex == 7 then
			BankGUIDeleteButton:SetText(BANKGUI_DELETE)
			BankGUIDeleteButton:Enable()
		else
			BankGUIDeleteButton:SetText(BANKGUI_DELETES)
			if BankUI.PlayerIndex == 1 then BankGUIDeleteButton:Disable()
			else BankGUIDeleteButton:Enable() end
		end
		if BankGUI.TypesIndex == 6 or BankGUI.TypesIndex == 8 then
			BankGUIDeleteButton:Disable()
		end
	end
	BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
end


function BankGUIPlayerDropDown_Initialize(self)
	local i, bank_gold, bank_gold_tot = 0, 0.0, 0.0
	local playermenuinfo = UIDropDownMenu_CreateInfo()
	if not BankListOptions then return end
	if not BankListOptions[BankGUI.realmname] then return end
	if not BankListOptions[BankGUI.realmname][1] then return end
	if BankListOptions[BankGUI.realmname][1].guild_checked then
		if not BankListGuildInfo then return end
		if not BankListGuildInfo[BankGUI.realmname] then return end
		if not BankListOptions[BankGUI.realmname][1].guild_name then
			BankListOptions[BankGUI.realmname][1].guild_name = BankGUI:GetGuild(1)
		end
		for i = 1, BankGUI:GetNumGuilds() do
			if BankListGuildInfo[BankGUI.realmname][i].money then
				if BankListGuildInfo[BankGUI.realmname][i].name == BankGUI:GetGuild(BankUI.PlayerIndex) then bank_gold_tot = (BankListGuildInfo[BankGUI.realmname][i].money/10000) end
				bank_gold = BankListGuildInfo[BankGUI.realmname][i].money/10000
			else bank_gold = 0.0 end
			playermenuinfo.text = format("%s (%.2f G)", BankListGuildInfo[BankGUI.realmname][i].name, bank_gold)
			playermenuinfo.value = BankListGuildInfo[BankGUI.realmname][i].name
			playermenuinfo.func = BankGUIPlayerDropDown_Select
			playermenuinfo.owner = self:GetParent()
			playermenuinfo.notCheckable = true
			playermenuinfo.checked = nil
			UIDropDownMenu_AddButton(playermenuinfo)
		end
		BankGUITotalGold:SetText( format(BANKGUI_GOLD_GUILD, bank_gold_tot) )
	else
		if not BankListPlayers then return end
		if not BankListPlayers[BankGUI.realmname] then return end
		playermenuinfo.text = "All Characters"
		playermenuinfo.value = "All Characters"
		playermenuinfo.func = BankGUIPlayerDropDown_Select
		playermenuinfo.owner = self:GetParent()
		playermenuinfo.notCheckable = true
		playermenuinfo.checked = nil
		UIDropDownMenu_AddButton(playermenuinfo)
		for i = 1, table.getn(BankListPlayers[BankGUI.realmname]) do
			if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankListPlayers[BankGUI.realmname][i].faction and BankListPlayers[BankGUI.realmname][i].faction == BankGUI.playerfaction) then
				if BankListPlayers[BankGUI.realmname][i].money then
					bank_gold_tot = bank_gold_tot + (BankListPlayers[BankGUI.realmname][i].money/10000)
					bank_gold = BankListPlayers[BankGUI.realmname][i].money/10000
				else
					bank_gold = 0.0
				end
				playermenuinfo.text = format("%s (%.2f G)", BankListPlayers[BankGUI.realmname][i].name, bank_gold)
				playermenuinfo.value = BankListPlayers[BankGUI.realmname][i].name
				playermenuinfo.func = BankGUIPlayerDropDown_Select
				playermenuinfo.owner = self:GetParent()
				playermenuinfo.checked = nil
				UIDropDownMenu_AddButton(playermenuinfo)
			end
		end
		BankGUITotalGold:SetText( format(BANKGUI_GOLD_BANK, bank_gold_tot) )
	end
end


function BankGUIPlayerDropDown_Select(self)
  BankUI.PlayerIndex = self:GetID()
  if BankListOptions[BankGUI.realmname][1].guild_checked then
	BankUI.SelectedID = BankGUI:GetGuild(BankUI.PlayerIndex)
	BankListOptions[BankGUI.realmname][1].guild_name = BankUI.SelectedID
	if BankUI.SelectedID == "" then
		BankGUITotalGold:SetText( format(BANKGUI_GOLD_GUILD,0.0) )
		BankGUIDeleteButton:Disable()
	else
		UIDropDownMenu_SetText(BankGUIPlayerDropDown, BankUI.SelectedID)
		BankGUITotalGold:SetText( format(BANKGUI_GOLD_GUILD,(BankListGuildInfo[BankGUI.realmname][BankUI.PlayerIndex].money/10000)) )
		BankGUIDeleteButton:Enable()
	end
	BankGUIDeleteButton:SetText(BANKGUI_DELETES)
  else
	if BankUI.PlayerIndex == 1 then
		BankUI.SelectedID = nil
		UIDropDownMenu_SetText(BankGUIPlayerDropDown, "All Characters")
		BankListOptions[BankGUI.realmname][1].player_name = nil
		if BankGUI.TypesIndex == 4 or BankGUI.TypesIndex == 5 then BankGUIDeleteButton:Enable()
		else BankGUIDeleteButton:Disable() end
	else
		local i, num_matchesfound = 0, 0
		for i = 1, table.getn(BankListPlayers[BankGUI.realmname]) do
			if (BankListOptions[BankGUI.realmname][1][BANKGUI_FACTION]) or (BankListPlayers[BankGUI.realmname][i].faction and BankListPlayers[BankGUI.realmname][i].faction == BankGUI.playerfaction) then
				num_matchesfound = num_matchesfound + 1
			end
			if num_matchesfound == BankUI.PlayerIndex-1 then
				BankUI.SelectedID = BankListPlayers[BankGUI.realmname][i].name
				break
			end
		end
		UIDropDownMenu_SetText(BankGUIPlayerDropDown, BankUI.SelectedID)
		BankGUIDeleteButton:Enable()
		BankListOptions[BankGUI.realmname][1].player_name = BankUI.SelectedID
	end
	if BankGUI.TypesIndex == 4 or BankGUI.TypesIndex == 5 then BankGUIDeleteButton:SetText(BANKGUI_DELETE)
	else BankGUIDeleteButton:SetText(BANKGUI_DELETES)
	end
	if BankGUI.TypesIndex == 6 or BankGUI.TypesIndex == 7 or BankGUI.TypesIndex == 8 then
		BankGUIDeleteButton:Disable()
	end
  end
  BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
end


function BankGUILinkDropDown_Initialize(self, level)
  local linkmenuinfo = UIDropDownMenu_CreateInfo()
  local i, friendsonline, name, lvl, class, area, connected = 0, false
  if level == nil then level = 1 end
  if level == 1 then
		linkmenuinfo.text = BankGUI:GetItemByIndex(BankGUI.ClickedIndex).item
		linkmenuinfo.value = nil
		linkmenuinfo.func = nil
		linkmenuinfo.owner = self:GetParent()
		linkmenuinfo.checked = nil
		linkmenuinfo.isTitle = true
		linkmenuinfo.hasArrow = false
		linkmenuinfo.nested = 0
		linkmenuinfo.notCheckable = true
		linkmenuinfo.disabled = true
		UIDropDownMenu_AddButton(linkmenuinfo, level)
		for i = 1, table.getn(BankUI.linktochattypes) do
			linkmenuinfo.text = BankUI.linktochattypes[i][1]
			linkmenuinfo.value = { ["Level1_Key"] = BankUI.linktochattypes[i][1] }
			linkmenuinfo.func = BankGUILinkDropDown_Select
			linkmenuinfo.owner = self:GetParent()
			linkmenuinfo.checked = nil
			linkmenuinfo.isTitle = BankUI.linktochattypes[i][3]
			linkmenuinfo.hasArrow = BankUI.linktochattypes[i][2]
			if linkmenuinfo.hasArrow then linkmenuinfo.nested = 1 else linkmenuinfo.nested = 0 end
			linkmenuinfo.notCheckable = true
			linkmenuinfo.disabled = nil
			if BankUI.linktochattypes[i][1] == "FRIEND" then
				_, friendsonline = GetNumFriends()
				if friendsonline <= 0 then linkmenuinfo.disabled = 1 end
			elseif BankUI.linktochattypes[i][1] == "WHISPERS" then
				if table.getn(BankGUI.Wisp_Data) == 0 then linkmenuinfo.disabled = 1 end
			elseif BankUI.linktochattypes[i][1] == "REALID" then
				_, friendsonline = BNGetNumFriends()
				if friendsonline <= 0 then linkmenuinfo.disabled = 1 end
			elseif BankUI.linktochattypes[i][1] == "RAID" then
				if GetNumGroupMembers() == 0 then linkmenuinfo.disabled = 1 end
				if UnitInRaid("player") == nil then linkmenuinfo.disabled = 1 end
			elseif BankUI.linktochattypes[i][1] == "PARTY" then
				if GetNumGroupMembers() == 0 then linkmenuinfo.disabled = 1 end
				if UnitInParty("player") == nil then linkmenuinfo.disabled = 1 end
			elseif BankUI.linktochattypes[i][1] == "GUILD" then
				if not GetGuildInfo("player") then linkmenuinfo.disabled = 1 end
			end
			UIDropDownMenu_AddButton(linkmenuinfo, level)
		end
  elseif level == 2 then
		ShowFriends()
		local friend, fname, lvl, class, area, connected
		local presenceID, givenName, surname
		local Level1_Key = UIDROPDOWNMENU_MENU_VALUE["Level1_Key"]
		BankUI.Link_Data[Level1_Key] = {}
		if Level1_Key == "FRIEND" then
			for i = 1, GetNumFriends() do
				fname, lvl, class, area, connected = GetFriendInfo(i)
				if name and connected then
					friend = { name = fname }
					table.insert(BankUI.Link_Data[Level1_Key], friend)
				end
			end
		elseif Level1_Key == "WHISPERS" then
			for i = 1, table.getn(BankGUI.Wisp_Data) do
				friend = BankGUI.Wisp_Data[i]
				table.insert(BankUI.Link_Data[Level1_Key], friend)
			end
		elseif Level1_Key == "REALID" then
			--[[for i = 1, table.getn(BankUI.RealID_Data) do
				friend = BankUI.RealID_Data[i]
				table.insert(BankUI.Link_Data[Level1_Key], friend)
			end--]]
			table.wipe(BankUI.RealID_Data)
			for i = 1, BNGetNumFriends() do
				presenceID, givenName, surname, _, _, _, _, connected = BNGetFriendInfo(i)
				if ( connected ) then
					friend = { name = givenName .." " ..surname, realid = presenceID }
					table.insert(BankUI.RealID_Data, friend)
				end
			end
			table.sort(BankUI.RealID_Data, function(a,b)
				if (a.name==nil or b.name==nil) then
					return false
				else
					return a.name < b.name
				end
			end )
			for i = 1, table.getn(BankUI.RealID_Data) do
				friend = BankUI.RealID_Data[i]
				table.insert(BankUI.Link_Data[Level1_Key], friend)
			end
		end
		for i = 1, table.getn(BankUI.Link_Data[Level1_Key]) do
			linkmenuinfo.text = BankUI.Link_Data[Level1_Key][i].name
			linkmenuinfo.value = { ["Level1_Key"] = Level1_Key, ["Sublevel_Key"] = BankUI.Link_Data[Level1_Key][i].name }
			if ( BankUI.Link_Data[Level1_Key][i].realid == nil ) then
				linkmenuinfo.func = BankGUIFriend_Select
			else
				linkmenuinfo.func = BankGUIRealid_Select
			end
			linkmenuinfo.owner = self:GetParent()
			linkmenuinfo.checked = nil
			linkmenuinfo.hasArrow = false
			linkmenuinfo.notCheckable = true
			linkmenuinfo.isTitle = false
			linkmenuinfo.disabled = false
			UIDropDownMenu_AddButton(linkmenuinfo, level)
		end
  end
end


function BankGUILinkDropDown_Select(self)
	local bankinfo = BankGUI:GetItemByIndex(BankGUI.ClickedIndex)
	if BankUI.linktochattypes[self:GetID()] and BankUI.linktochattypes[self:GetID()][2] == nil then
		if bankinfo.type == BANKGUI_TYPE_TRADELINK then
			SendChatMessage( format(BANKGUI_TITLE ..": %s %s (%s)", bankinfo.name, bankinfo.item, bankinfo.amount), BankUI.linktochattypes[self:GetID()][1], nil)
		else
			SendChatMessage(bankinfo.item, BankUI.linktochattypes[self:GetID()][1], nil)
		end
	end
	CloseDropDownMenus()
end


function BankGUIFriend_Select(self)
	local bankinfo = BankGUI:GetItemByIndex(BankGUI.ClickedIndex)
	if bankinfo.type == BANKGUI_TYPE_TRADELINK then
		SendChatMessage( format(BANKGUI_TITLE ..": %s %s (%s)", bankinfo.name, bankinfo.item, bankinfo.amount), "WHISPER", nil, UIDropDownMenuButton_GetName(self))
	else
		SendChatMessage(bankinfo.item, "WHISPER", nil, UIDropDownMenuButton_GetName(self))
	end
	CloseDropDownMenus()
end


function BankGUIRealid_Select(self)
	local bankinfo = BankGUI:GetItemByIndex(BankGUI.ClickedIndex)
	if bankinfo.type == BANKGUI_TYPE_TRADELINK then
		BNSendWhisper(BankUI.RealID_Data[self:GetID()].realid, format(BANKGUI_TITLE ..": %s %s (%s)", bankinfo.name, bankinfo.item, bankinfo.amount))
	else
		BNSendWhisper(BankUI.RealID_Data[self:GetID()].realid, bankinfo.item)
	end
	CloseDropDownMenus()
end


function BankGUIDetailsIcon_OnClick(button)
	if (button == "LeftButton") then
		local bankinfo = BankGUI:GetItemByIndex(BankUI.SelectedIndex)
		if IsShiftKeyDown() then
			if BankGUI.chat:IsVisible() then
				BankGUI.chat.editBox:SetFocus()
				BankGUI.chat.editBox:SetText(bankinfo.item)
			else BankGUI.chat:AddMessage(bankinfo.item)
			end
		elseif (IsControlKeyDown() and IsDressableItem(bankinfo.item)) then DressUpItemLink(bankinfo.item)
		else BankGUI.chat:AddMessage(bankinfo.item)
		end
	end
end


function BankGUIDetailsIcon_OnEnter(self)
	local bankinfo = BankGUI:GetItemByIndex(BankUI.SelectedIndex)
	GameTooltip:SetOwner(self,"ANCHOR_BOTTOMRIGHT")
	GameTooltip:ClearLines()
	GameTooltip:SetHyperlink(bankinfo.item)
end


local bank_farm_player = UnitName("player")

function BankGUIFarmFrame_OnShow()
	UIDropDownMenu_SetText(BankGUIFarmPlayerDropDown, bank_farm_player)
	BankGUIFarmItemBox:SetFocus()
end


function BankGUIFarmPlayerDropDown_Initialize(self)
	local i, k, farmmenuinfo = 0, 0, UIDropDownMenu_CreateInfo()
	if not BankListPlayers then return end
	if not BankListPlayers[BankGUI.realmname] then return end
	for i = 1, table.getn(BankListPlayers[BankGUI.realmname]) do
		farmmenuinfo.text = BankListPlayers[BankGUI.realmname][i].name
		farmmenuinfo.value = BankListPlayers[BankGUI.realmname][i].name
		farmmenuinfo.func = BankGUIFarmPlayerDropDown_Select
		farmmenuinfo.owner = self:GetParent()
		farmmenuinfo.checked = nil
		farmmenuinfo.notCheckable = true
		UIDropDownMenu_AddButton(farmmenuinfo)
 	end
end


function BankGUIFarmPlayerDropDown_Select(self)
	bank_farm_player = self.value
	UIDropDownMenu_SetText(BankGUIFarmPlayerDropDown, bank_farm_player)
end


local bank_farm_item

function BankGUIFarmItemBox_OnTextChanged(self)
	CloseDropDownMenus()
	bank_farm_item = self:GetText()
	if string.len(bank_farm_item) == 0 then return end
	BankGUIFarmItemDropDown.initialize = BankGUIFarmItemDropDown_Initialize
	BankGUIFarmItemDropDown.displayMode = "MENU"
	ToggleDropDownMenu(1, nil, BankGUIFarmItemDropDown, "BankGUIFarmItemBox", 0, 0)
end


function BankGUIFarmItemDropDown_Initialize(self)
	local i, k, itemName = 0, 0, ""
	local farmitemmenuinfo = UIDropDownMenu_CreateInfo()
	for i = 1, table.getn(BankListItems[BankGUI.realmname]) do
		itemName = GetItemInfo(BankListItems[BankGUI.realmname][i].item)
		if itemName and k < 25 and BankListItems[BankGUI.realmname][i].bank and string.find(string.lower(itemName), bank_farm_item) ~= nil then
			farmitemmenuinfo.text = BankListItems[BankGUI.realmname][i].item
			farmitemmenuinfo.value = i
			farmitemmenuinfo.func = BankGUIFarmItemDropDown_Select
			farmitemmenuinfo.owner = self:GetParent()
			farmitemmenuinfo.checked = nil
			farmitemmenuinfo.hasArrow = false
			farmitemmenuinfo.notCheckable = true
			farmitemmenuinfo.isTitle = false
			farmitemmenuinfo.disabled = nil
			UIDropDownMenu_AddButton(farmitemmenuinfo)
			k = k + 1
		end
 	end
end


function BankGUIFarmItemDropDown_Select(self)
	BankGUIFarmItemBox:SetText("")
	BankGUIFarmItemBox:Insert(BankListItems[BankGUI.realmname][self.value].item)
end


function BankGUIFarmAddItem_OnClick()
	local farminfo, itemname, itemlink, itemstack = {}
	itemname, itemlink = GetItemInfo(BankGUIFarmItemBox:GetText())
	farminfo.name = bank_farm_player
	if itemlink then farminfo.item = itemlink
	else farminfo.item = BankGUIFarmItemBox:GetText() end
	farminfo.amount = BankGUIFarmAmountBox:GetNumber()
	farminfo.date = date("%d/%m/%y")
	if farminfo.item ~= "" and farminfo.amount > 0 then
		PlaySound("igMainMenuOptionCheckBoxOn")
		table.insert(BankListFarms[BankGUI.realmname], farminfo)
		itemname, _, _, _, _, _, _, itemstack = GetItemInfo(farminfo.item)
		if not itemname then
			itemname = BankGUIFarmItemBox:GetText()
			itemstack = 20
		end
		if itemname and itemstack > 1 then
			if not BankGUI.BankListStock[itemname] then BankGUI.BankListStock[itemname] = {} end
			if not BankGUI.BankListStock[itemname].farm then BankGUI.BankListStock[itemname].farm = {} end
			if not BankGUI.BankListStock[itemname].farm.amount then BankGUI.BankListStock[itemname].farm.amount = 0 end
			if not BankGUI.BankListStock[itemname].farm.date then BankGUI.BankListStock[itemname].farm.date = date("%d/%m/%y") end
			BankGUI.BankListStock[itemname].farm.amount = BankGUI.BankListStock[itemname].farm.amount + farminfo.amount
			BankGUI.BankListStock[itemname].farm.name   = farminfo.name
		end
		BankGUI:UpdateFilter(BankGUIFilterTextBox:GetText())
	end
	BankGUIFarmFrame:Hide()
end


local itema, itemb, itemalvl, itemblvl, itemamin, itembmin, amounta, amountb, sortvala, sortvalb, sortstringa, sortstringb

local function BankGUI_GetSortValue1(sortvalue, sortvalueA, sortvalueB, a, b)
	if sortvalue == BANKGUI_SORTNAME1 then
		if itema == itemb then
			return sortvalueA, sortvalueB
		elseif itema < itemb then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	elseif sortvalue == BANKGUI_SORTNAME2 then
		if a.name == b.name then
			return sortvalueA, sortvalueB
		elseif a.name < b.name then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	elseif sortvalue == BANKGUI_SORTNAME3 then
		if amounta == amountb then
			return sortvalueA, sortvalueB
		elseif amounta > amountb then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	elseif sortvalue == BANKGUI_SORTNAME4 then
		if (itemalvl == itemblvl) and (itemamin == itembmin) then
			return sortvalueA, sortvalueB
		elseif (itemalvl > itemblvl) or (itemalvl == itemblvl and itemamin > itembmin) then
			sortvalueA = sortvalueA + 4
		else
			sortvalueB = sortvalueB + 4
		end
	end
	return sortvalueA, sortvalueB
end


local function BankGUI_GetSortValue2(sortvalue, sortvalueA, sortvalueB, a, b)
	if sortvalue == BANKGUI_SORTNAME1 then
		if itema == itemb then
			return sortvalueA, sortvalueB
		elseif itema < itemb then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	elseif sortvalue == BANKGUI_SORTNAME2 then
		if a.name == b.name then
			return sortvalueA, sortvalueB
		elseif a.name < b.name then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	elseif sortvalue == BANKGUI_SORTNAME3 then
		if amounta == amountb then
			return sortvalueA, sortvalueB
		elseif amounta > amountb then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	elseif sortvalue == BANKGUI_SORTNAME4 then
		if (itemalvl == itemblvl) and (itemamin == itembmin) then
			return sortvalueA, sortvalueB
		elseif (itemalvl > itemblvl) or (itemalvl == itemblvl and itemamin > itembmin) then
			sortvalueA = sortvalueA + 2
		else
			sortvalueB = sortvalueB + 2
		end
	end
	return sortvalueA, sortvalueB
end


local function BankGUI_GetSortValue3(sortvalue, sortvalueA, sortvalueB, a, b)
	if sortvalue == BANKGUI_SORTNAME1 then
		if itema == itemb then
			return sortvalueA, sortvalueB
		elseif itema < itemb then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	elseif sortvalue == BANKGUI_SORTNAME2 then
		if a.name == b.name then
			return sortvalueA, sortvalueB
		elseif a.name < b.name then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	elseif sortvalue == BANKGUI_SORTNAME3 then
		if amounta == amountb then
			return sortvalueA, sortvalueB
		elseif amounta > amountb then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	elseif sortvalue == BANKGUI_SORTNAME4 then
		if (itemalvl == itemblvl) and (itemamin == itembmin) then
			return sortvalueA, sortvalueB
		elseif (itemalvl > itemblvl) or (itemalvl == itemblvl and itemamin > itembmin) then
			sortvalueA = sortvalueA + 1
		else
			sortvalueB = sortvalueB + 1
		end
	end
	return sortvalueA, sortvalueB
end


function BankGUI_SortTotals()
	if BankListOptions[BankGUI.realmname][1][BANKGUI_TOTALS] == true then
		for subarray in pairs(BankGUI.BankTotals) do
			for subsubarray in pairs(BankGUI.BankTotals[subarray]) do
				bankinfo = {}
				for subsubsub in pairs(BankGUI.BankTotals[subarray][subsubarray]) do
					bankinfo[subsubsub] = BankGUI.BankTotals[subarray][subsubarray][subsubsub]
				end
				table.insert(BankGUI.BankFilter, bankinfo)
			end
		end
	end
	if  BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE1].name == BANKGUI_SORTNAME5
	and BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE2].name == BANKGUI_SORTNAME5
	and BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE3].name == BANKGUI_SORTNAME5 then
		return
	end
	table.sort(BankGUI.BankFilter,
	function(a, b)
		local i
		itema, _, _, itemalvl, itemamin = GetItemInfo(a.item)
		itemb, _, _, itemblvl, itembmin = GetItemInfo(b.item)
		if itema == nil then
			if a.type == BANKGUI_TYPE_TRADELINK then
				i = string.find(a.item, "|Htrade")
				if i == nil then itema = a.item
				else
					i = string.find(a.item, "|h", i)
					i = i + 3
					itema = string.sub(a.item, i, string.len(a.item) - 5)
				end
			else
				itema = a.item
			end
		end
		if itemb == nil then
			if b.type == BANKGUI_TYPE_TRADELINK then
				i = string.find(b.item, "|Htrade")
				if i == nil then itemb = b.item
				else
					i = string.find(b.item, "|h", i)
					i = i + 3
					itemb = string.sub(b.item, i, string.len(b.item) - 5)
				end
			else
				itemb = b.item
			end
		end
		if itemalvl == nil then itemalvl = 0 end
		if itemblvl == nil then itemblvl = 0 end
		if itemamin == nil then itemamin = 0 end
		if itembmin == nil then itembmin = 0 end
		amounta = tonumber(a.amount)
		amountb = tonumber(b.amount)
		if amounta == nil then amounta = 0 end
		if amountb == nil then amountb = 0 end
		sortvala = 0 sortvalb = 0
		if BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE1].name ~= BANKGUI_SORTNAME5 then
			sortvala, sortvalb = BankGUI_GetSortValue1(BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE1].name, sortvala, sortvalb, a, b)
		end
		if BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE2].name ~= BANKGUI_SORTNAME5 then
			sortvala, sortvalb = BankGUI_GetSortValue2(BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE2].name, sortvala, sortvalb, a, b)
		end
		if BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE3].name ~= BANKGUI_SORTNAME5 then
			sortvala, sortvalb = BankGUI_GetSortValue3(BankListOptions[BankGUI.realmname][1][BANKGUI_SORTTITLE3].name, sortvala, sortvalb, a, b)
		end
		return sortvala > sortvalb
	end )
end


function BankGUI_Update()
	BankGUI:UpdateUI()
end
